#ifdef __linux__

#include "stdafx.h"

#else

#include <string>
#include <exception>
#include <Isolator.h>
#include <gtest/gtest.h>
using namespace std;

#endif

#include "../ClassesUnderTest/Person.h"

class ArgumentTests : public ::testing::Test
{
public:
	virtual void TearDown()
	{
		ISOLATOR_CLEANUP();
	}

};

/*****************************************
		Not specifying parameters
******************************************/

/*	Isolator++ ignores arguments by default. Although we need
	to specify args in the signature, it's just for compilation reasons. 
	The _ macro spare us from writing them for simple types. It can
	be also used for assertion methods.
*/
TEST_F(ArgumentTests, UsingThe_MacroForShorthand)
{
	Person person;
	Address address;

	WHEN_CALLED(address.SetStreetName(_)).Ignore();

	person.SetStreetNameOnAddress(&address, "HighStreet");

	string result = person.GetStreetNameFromAddress(&address);

	ASSERT_TRUE(result.compare("") == 0);
}

/*	When the method has more complex types, passed on the stack,	
	We can use the ANY_VAL macro, specifying the type of the argument.
	It can also be used in the assertion methods.
*/
TEST_F(ArgumentTests, UsingTheANY_VALMacroForShorthand)
{
	Person person;
	Address address;

	WHEN_CALLED(address.SetCountry(ANY_VAL(string))).Ignore();

	person.SetAddressCountry(&address, "UK");
	ASSERT_WAS_CALLED(address.SetCountry(ANY_VAL(string)));
}

/*	When the method has an argument, passed by reference,	
	We can use the ANY_REF macro, specifying the type of the argument.
*/
TEST_F(ArgumentTests, UsingTheANY_REFMacroForShorthand)
{
	Person person;
	Address address;

	WHEN_CALLED(address.SetCountry(ANY_VAL(string))).Ignore();

	person.SetAddressCountry(&address, "UK");
	ASSERT_WAS_CALLED(address.SetCountry(ANY_VAL(string)));
}


/*****************************************
		Faking referenced parameters
******************************************/

/*	It is possible to simulate arguments that are returned by reference using the 
	RET macro. You can use RET for every referenced argument 
*/
TEST_F(ArgumentTests, UsingRETToSimulateReturnedValue)
{
	Person person;
	Address* fakeAddress = FAKE<Address>();
	string fakeStreet("HighStreet");

	WHEN_CALLED(fakeAddress->GetStreetName(RET(&fakeStreet))).Ignore();

	string result = person.GetStreetNameFromAddress(fakeAddress);

	ASSERT_TRUE(result.compare(fakeStreet) == 0);
}


/*****************************************
		Argument Validation
******************************************/

// Struct for storing validation result
struct Validator
{
public:
	bool result;
};

// Method for argument validation
std::string  CALLBACK ValidateArgumentIsLessThanFiveChars(string country)
{
	Validator* validator = (Validator*)GET_EXTRA_DATA();
	if (country.length() < 5)
		validator->result = true;
	else
		validator->result = false;

	return "";
}

/*	We can use the DoInstead methods to do specific argument validation. In this
	example, we're passing to the DoStaticOrGlobalInstead method an extra parameter, that
	we create and examine in the test. The validation method can extract and modify it with
	the GET_EXTRA_DATA macro. We can then validate the arguments in the test.
	
*/
TEST_F(ArgumentTests, ValidatingArgumentInSeparateMethod)
{
	Person person;
	Address* address = FAKE<Address>();

	Validator validator;
	validator.result = false;

	WHEN_CALLED(address->GetCityByCountry(ANY_VAL(string))).
		DoStaticOrGlobalInstead(ValidateArgumentIsLessThanFiveChars, &validator);

	person.GetCityFromAddress(address, "UK");
	ASSERT_TRUE(validator.result);

	person.GetCityFromAddress(address, "Germany");
	ASSERT_FALSE(validator.result);

	
}
