#ifdef __linux__

#include "stdafx.h"

#else

#include <string>
#include <exception>
#include <Isolator.h>
#include <gtest/gtest.h>
using namespace std;

#endif

#include "../ClassesUnderTest/Person.h"

class AssertingInteraction : public ::testing::Test
{
public:
	virtual void TearDown()
	{
		ISOLATOR_CLEANUP();
	}
};

/*****************************************
		Method Interaction Assertion
******************************************/

/*
 * This test shows the usage of ASSERT_WAS_CALLED macro for a method called on a faked dependency.
 * If pAddress->SetCountry will not be called Isolator++ will throw a TypemockException
*/
TEST_F(AssertingInteraction, AssertCallWasMadeOnDependency)
{
	Person person;
	Address* pAddress = FAKE<Address>();

	person.SetAddressCountry(pAddress, "US"); 
	
	ASSERT_WAS_CALLED(pAddress->SetCountry(""));
}

// If there is no lambda support you can use this predicate
struct MatchStringPredicateUK
{
    bool operator ()(const string &s) { return !s.compare("UK"); }
};


/*
 * This test shows the usage of ASSERT_WAS_CALLED macro for a method called with a specific argument on a faked dependency.
 * If pAddress->SetCountry will not be called with "UK" Isolator++ will throw a TypemockException
*/
TEST_F(AssertingInteraction, AssertCallWasMadeWithSpecificArgumentsOnDependency)
{
	Person person;
	Address* pAddress = FAKE<Address>();

	person.SetAddressCountry(pAddress, "UK"); 
	
#ifdef NO_CPP_LAMBDA_SUPPORT
        ASSERT_WAS_CALLED(pAddress->SetCountry(IS<string>(MatchStringPredicateUK())));
#else
        ASSERT_WAS_CALLED(pAddress->SetCountry(IS<string>([](const string& s) { return !s.compare("UK"); })));
#endif
}

TEST_F(AssertingInteraction, AssertCallWasMadeWithSpecificArgumentsOnDependencyWithPredicates)
{
	Person person;
	Address* pAddress = FAKE<Address>();

	person.SetAddressCountry(pAddress, "UK"); 

#ifdef NO_CPP_LAMBDA_SUPPORT
	ASSERT_WAS_CALLED(pAddress->SetCountry(IS<string>(MatchStringPredicateUK())));
#else
	ASSERT_WAS_CALLED(pAddress->SetCountry(IS<string>([](const string& str){return !str.compare("UK");})));
#endif
}

/*
 * This test shows the usage of ASSERT_NOT_CALLED macro for a method called on a faked dependency.
 * If pAddress->SetCountry is called, Isolator++ will throw a TypemockException
*/
TEST_F(AssertingInteraction, AssertCallWasNotMadeOnDependency)
{
	Address* pAddress = FAKE<Address>();
	ASSERT_NOT_CALLED(pAddress->SetCountry(""));
}

/*
 * This test shows the usage of TIMES_CALLED macro 
 * Isolator++ keeps track of the called methods in the test
 * Using TIMES_CALLED you can verify how many times a method got called in the test.
*/
TEST_F(AssertingInteraction, TimesCalledMethodOnDependency)
{
	Person person;
	Address* pAddress = FAKE<Address>();

	person.SetAddressCountry(pAddress, "US"); 
	person.SetAddressCountry(pAddress, "UK"); 
	person.SetAddressCountry(pAddress, "Germany"); 
	
	int timesCalled = TIMES_CALLED(pAddress->SetCountry(""));
	ASSERT_EQ(3, timesCalled);
}

/*
 * This test shows how to handle methods with out/by-ref value type arguments
 * Note, as GetOlder method has int* argument we can't just write EQ(2)
 * Using BY_REF there is no need to define another variable for assertion.
 * Using ASSERT_NOT_CALLED you can verify a method wasn't called in the test with specific arguments.
*/
TEST_F(AssertingInteraction, WasNotCalledWithConcreteOutParam)
{
	Person* person = FAKE<Person>(FakeOptions::CallOriginal); // Can also use FakeOptions::Lean, but methods will not be tracked and asserting them will require using WHEN_CALLED on the method

	int age = 0;
	person->GetOlder(&age);
	person->GetOlder(&age);

	ASSERT_NOT_CALLED(person->GetOlder(EQ(BY_REF(2))));
}

/*
 * This test shows how to handle methods with out/by-ref value type arguments
 * Note, as GetOlder method has int* argument we can't just write EQ(0)
 * Using BY_REF there is no need to define another variable for assertion.
 * Using ASSERT_WAS_CALLED you can verify a method was called in the test with specific arguments.
*/
TEST_F(AssertingInteraction, WasCalledWithConcreteOutParam)
{
	Person* person = FAKE<Person>(FakeOptions::CallOriginal); // Can also use FakeOptions::Lean, but methods will not be tracked and asserting them will require using WHEN_CALLED on the method

	int age = 0;
	person->GetOlder(&age);
	person->GetOlder(&age);

	ASSERT_WAS_CALLED(person->GetOlder(EQ(BY_REF(0))));
}
