#ifdef __linux__

#include "stdafx.h"

#else

#include <string>
#include <exception>
#include <Isolator.h>
#include <gtest/gtest.h>
using namespace std;

#endif

#include "../ClassesUnderTest/Person.h"

class AssertingNonPublicInteraction : public ::testing::Test
{
public:
	virtual void TearDown()
	{
		ISOLATOR_CLEANUP();
	}
};

/*****************************************
		Method Interaction Assertion
******************************************/

/*
 * This test shows the usage of PRIVATE_ASSERT_WAS_CALLED macro for a method called on a faked dependency.
 * If pAddress->SetCountryInternal is not be called Isolator++ will throw a TypemockException
*/
TEST_F(AssertingNonPublicInteraction, AssertCallWasMadeOnDependency)
{
	Person person;
	Address* pAddress = FAKE<Address>(FakeOptions::CallOriginal); // Can also use FakeOptions::Lean, but methods will not be tracked and asserting them will require using WHEN_CALLED on the method

	person.SetAddressCountryInternal(pAddress, "US"); 
	
	PRIVATE_ASSERT_WAS_CALLED(pAddress, SetCountryInternal);
}


/*
 * This test shows the usage of PRIVATE_ASSERT_NOT_CALLED macro for a method called on a faked dependency.
 * If Address::CountReference is called, Isolator++ will throw a TypemockException
*/
TEST_F(AssertingNonPublicInteraction, AssertCallWasNotMadeOnStaticDependency)
{
	Person person;
	Address* pAddress = FAKE<Address>();

	FAKE_STATICS<Address>();

	person.SetAddressCountry(pAddress, "US"); 

	// this is how we test static methods
	PRIVATE_ASSERT_NOT_CALLED(_, Address::CountReference);
}

/*
 * This test shows the usage of PRIVATE_TIMES_CALLED macro 
 * Isolator++ keeps track of the called methods in the test
 * Using PRIVATE_TIMES_CALLED you can verify how many times a method got called in the test.
*/
TEST_F(AssertingNonPublicInteraction, TimesCalledMethodOnDependency)
{
	FAKE_STATICS<Address>();
	Person person;
	Address* pAddress = FAKE<Address>();

	person.SetAddressCountry(pAddress, "US"); 
	person.SetAddressCountry(pAddress, "UK"); 
	person.SetAddressCountry(pAddress, "Germany"); 
	
	int timesCalled = PRIVATE_TIMES_CALLED(pAddress, SetCountry); // we can use the private api on a public method
	ASSERT_EQ(3, timesCalled);
}

/*
 * This test shows the usage of PRIVATE_ASSERT_WAS_CALLED macro for an overloaded method called on a faked dependency.
 * If pAddress->OverloadedPrivate(int) is not be called or pAddress->OverloadedPrivate(char &) is called Isolator++ will throw a TypemockException
*/
TEST_F(AssertingNonPublicInteraction, AssertOverloadCallWasMadeOnDependency)
{
	Person person;
	Address* pAddress = FAKE<Address>(FakeOptions::CallOriginal); // Can also use FakeOptions::Lean, but methods will not be tracked and asserting them will require using WHEN_CALLED on the method

	person.CallAddressOverloadedPrivate(pAddress); 
	
	PRIVATE_ASSERT_WAS_CALLED(pAddress, OverloadedPrivate, TYPEOF(int));
	PRIVATE_ASSERT_NOT_CALLED(pAddress, OverloadedPrivate, TYPEOF(char *));
}

/*
 * This test shows the usage of PRIVATE_ASSERT_WAS_CALLED macro for a non public method called with a specific arguments.
 * If person->SaveAll(string) is not be called with "foo" as an argument Isolator++ will throw a TypemockException
*/
#if defined(__linux__) && !defined(__clang__)
struct MatchStringPredicateFoo
{
    bool operator ()(const string &s) { return !s.compare("foo"); }
};
#endif

TEST_F(AssertingNonPublicInteraction, AssertPrivateWithCondition)
{
	Person* person = new Person();

	PRIVATE_WHEN_CALLED(person, IsValidName).Return(true);
	PRIVATE_WHEN_CALLED(person, SaveAll).Ignore();

	person->Save("foo");

#if defined(__linux__) && !defined(__clang__)
        PRIVATE_ASSERT_WAS_CALLED(person, SaveAll, IS<std::string>(MatchStringPredicateFoo()));
#else
	PRIVATE_ASSERT_WAS_CALLED(person, SaveAll, IS<std::string>([](const string& val)
         {
             return val == "foo";
         }));
#endif
}

/*
 * This test shows the usage of PRIVATE_ASSERT_NOT_CALLED macro for a non public method called with a specific arguments.
 * If person->SaveAll(string) was called with "foo" as an argument Isolator++ will throw a TypemockException
*/
TEST_F(AssertingNonPublicInteraction, AssertPrivateWithConditionWasNotCalled)
{
	Person* person = new Person();

	PRIVATE_WHEN_CALLED(person, IsValidName).Return(true);
	PRIVATE_WHEN_CALLED(person, SaveAll).Ignore();

	person->Save("bar");

#if defined(__linux__) && !defined(__clang__)
        PRIVATE_ASSERT_NOT_CALLED(person, SaveAll, IS<std::string>(MatchStringPredicateFoo()));
#else
        auto predicate = [](const std::string& val)
        {
            return val == "foo";
        };
	PRIVATE_ASSERT_NOT_CALLED(person, SaveAll, IS<std::string>(predicate));
#endif
}

/*
 * This test shows the usage of PRIVATE_ASSERT_WAS_CALLED macro for a non public method called with in/out arguments.
 * If person->ChangeLastName(char** otherLastName) is not be called with "Smith" as an argument Isolator++ will throw a TypemockException
*/
TEST_F(AssertingNonPublicInteraction, AssertPrivateWithConditionWasCalledWithRet)
{
	Person* person = FAKE<Person>(FakeOptions::CallOriginal); // this create a real object - we can track it for assert
															// Can also use FakeOptions::Lean, but methods will not be tracked and asserting them will require using WHEN_CALLED on the method

	Person* anotherPerson = new Person();
	anotherPerson->lastName = "Smith";

	person->GetMarried(anotherPerson);

	PRIVATE_ASSERT_WAS_CALLED(person, ChangeLastName, EQ("Smith")); // even though it was called with Smith
}

/*
 * This test shows the usage of PRIVATE_ASSERT_NOT_CALLED macro for a non public method called with in/out arguments.
 * Only if the method is called with a "Brown" PRIVATE_ASSERT_NOT_CALLED will throw an exception.
*/
TEST_F(AssertingNonPublicInteraction, AssertPrivateWithConditionWasNotCalledWithRet)
{
	Person* person = FAKE<Person>(FakeOptions::CallOriginal); // this create a real object - we can track it for assert
															// Can also be faked with FakeOptions::Lean, but asserting methods will require using WHEN_CALLED on the method

	Person* anotherPerson = new Person();
	anotherPerson->lastName = "Smith";

	person->GetMarried(anotherPerson);

	PRIVATE_ASSERT_NOT_CALLED(person, ChangeLastName, EQ("Brown")); // even though it was not called with Brown
}
