#ifdef __linux__

#include "stdafx.h"

#else

#include <string>
#include <exception>
#include <Isolator.h>
#include <gtest/gtest.h>
using namespace std;

#endif

#include "Zombieland.h"
#include "../ClassesUnderTest/Person.h"


// Conditional arguments changes behavior according to arguments passed
class Conditional : public ::testing::Test
{
public:
	virtual void TearDown()
	{
		ISOLATOR_CLEANUP();
	}

};

/*******************************************
	Setting custom behavior with Conditional Arguments
********************************************/
	
/*  Sometimes, we need to supply different behavior based on the parameters that 
	are passed to the method at runtime. In this example, we're returning different 
	values based on different method calls, when the faked method is static.
*/
TEST_F(Conditional, ReplacingStaticMethodCallBasedOnParameters)
{

	WHEN_CALLED(Address::GetZipCode(EQ("US"),EQ("NY"))).Return("12345");
	WHEN_CALLED(Address::GetZipCode(_,_)).Return("00000");

	auto result= Address::GetZipCode("UK","LONDON");
	auto result2= Address::GetZipCode("US","NY");

	ASSERT_TRUE(strcmp(result,"00000")==0);
	ASSERT_TRUE(strcmp(result2,"12345")==0);

}

/*  In this example, we're returning different 
        values using a custom condition.
*/

// If there is no lambda support you can use this predicate
struct MatchStringPredicateUKorUS
{
    bool operator ()(const string &s) { return !s.compare("UK") || !s.compare("US"); }
};

struct MatchStringPredicateUK
{
    bool operator ()(const string &s) { return !s.compare("UK"); }
    bool operator ()(char* s) { return !strcmp(s, "UK"); }
    bool operator ()(const char*& s) { return !strcmp(s, "UK"); }
};

struct MatchStringPredicateUS
{
    bool operator ()(const string &s) { return !s.compare("US"); }
    bool operator ()(char* s) { return !strcmp(s, "US"); }
    bool operator ()(const char*& s) { return !strcmp(s, "US"); }
};

struct MatchStringPredicateNY
{
    bool operator ()(const string &s) { return !s.compare("NY"); }
    bool operator ()(char* s) { return !strcmp(s, "NY"); }
    bool operator ()(const char*& s) { return !strcmp(s, "NY"); }
};



/*  The same test can be done with Predicates instead of EQ
    Useful in cases where you want to make sure a value that fits
    Some parameters but doesn't have to be equal
*/
#ifdef NO_CPP_LAMBDA_SUPPORT
TEST_F(Conditional, ReplacingStaticMethodCallBasedOnParametersWithPredicates)
{

	WHEN_CALLED(Address::GetZipCode(IS<char*>(MatchStringPredicateUS())
                                        , IS<char*>(MatchStringPredicateNY()))).Return("12345");
	WHEN_CALLED(Address::GetZipCode(_, _)).Return("00000");

	auto result = Address::GetZipCode("UK", "LONDON");
	auto result2 = Address::GetZipCode("US", "NY");

	ASSERT_TRUE(strcmp(result,"00000")==0);
	ASSERT_TRUE(strcmp(result2,"12345")==0);
}

TEST_F(Conditional, ReplacingMethodCallBasedOnParameters)
{
	Person person;
	Address* address = FAKE<Address>();
	Zombieland land;
	std::string city1("Unknown");
	std::string cityYork("York");

    WHEN_CALLED(address->GetCityByCountry(
        IS<std::string>(MatchStringPredicateUKorUS()
    ))).Return(&cityYork);	
    WHEN_CALLED(address->GetCityByCountry(ANY_VAL(std::string))).Return(&city1);

	string firstCity  = person.GetCityFromAddress(address, "US");
	string secondCity = person.GetCityFromAddress(address, "SP");
	string thirdCity  = person.GetCityFromAddress(address, "UK");
	string fourthCity = person.GetCityFromAddress(address, "CA");

	ASSERT_EQ(0, firstCity.compare("York"));
	ASSERT_EQ(0, secondCity.compare("Unknown"));
	ASSERT_EQ(0, thirdCity.compare("York"));
	ASSERT_EQ(0, fourthCity.compare("Unknown"));
}

TEST_F(Conditional, ReplacingPrivateMethodCallBasedOnParameters)
{
	Person person;
	Address* address = FAKE<Address>();
	Zombieland land;
	std::string city1("Unknown");
	std::string cityYork("York");

	// public methods can be set as private methods too
    PRIVATE_WHEN_CALLED(address, GetCityByCountry,
        IS<std::string>(MatchStringPredicateUKorUS()
    )).Return(&cityYork);
	
    PRIVATE_WHEN_CALLED(address, GetCityByCountry(ANY_VAL(std::string))).Return(&city1);

	string firstCity = person.GetCityFromAddress(address, "US");
	string secondCity = person.GetCityFromAddress(address, "UK");
	string thirdCity = person.GetCityFromAddress(address, "CA");

	ASSERT_EQ(0, firstCity.compare("York"));
	ASSERT_EQ(0, secondCity.compare("York"));
	ASSERT_EQ(0, thirdCity.compare("Unknown"));
}

#else //use lambda
TEST_F(Conditional, ReplacingStaticMethodCallBasedOnParametersWithPredicates)
{

    WHEN_CALLED(Address::GetZipCode(IS<char*>([](char* str) { return !strcmp(str,"US"); }) , IS<char*>([](char* str) { return !strcmp(str,"NY"); }))).Return("12345");
	WHEN_CALLED(Address::GetZipCode(_, _)).Return("00000");

	auto result = Address::GetZipCode("UK", "LONDON");
	auto result2 = Address::GetZipCode("US", "NY");

	ASSERT_TRUE(strcmp(result,"00000")==0);
	ASSERT_TRUE(strcmp(result2,"12345")==0);
}

TEST_F(Conditional, ReplacingMethodCallBasedOnParameters)
{
	Person person;
	Address* address = FAKE<Address>();
	Zombieland land;
	std::string city1("Unknown");
	std::string cityYork("York");

	WHEN_CALLED(address->GetCityByCountry(
        IS<std::string>([](const std::string& country) { return country.compare("US") == 0 || country.compare("UK") == 0; }
    ))).Return(&cityYork);
    WHEN_CALLED(address->GetCityByCountry(ANY_VAL(std::string))).Return(&city1);

	string firstCity  = person.GetCityFromAddress(address, "US");
	string secondCity = person.GetCityFromAddress(address, "SP");
	string thirdCity  = person.GetCityFromAddress(address, "UK");
	string fourthCity = person.GetCityFromAddress(address, "CA");

	ASSERT_EQ(0, firstCity.compare("York"));
	ASSERT_EQ(0, secondCity.compare("Unknown"));
	ASSERT_EQ(0, thirdCity.compare("York"));
	ASSERT_EQ(0, fourthCity.compare("Unknown"));
}

/*  In this example, we're returning different
values using a custom condition using the non public api.
*/
TEST_F(Conditional, ReplacingPrivateMethodCallBasedOnParameters)
{
	Person person;
	Address* address = FAKE<Address>();
	Zombieland land;
	std::string city1("Unknown");
	std::string cityYork("York");

	// public methods can be set as private methods too
    PRIVATE_WHEN_CALLED(address, GetCityByCountry, 
        IS<std::string>([](const std::string& country) { return country.compare("US") == 0 || country.compare("UK") == 0; }
    )).Return(&cityYork);	

	
    PRIVATE_WHEN_CALLED(address, GetCityByCountry(ANY_VAL(std::string))).Return(&city1);

	string firstCity = person.GetCityFromAddress(address, "US");
	string secondCity = person.GetCityFromAddress(address, "UK");
	string thirdCity = person.GetCityFromAddress(address, "CA");

	ASSERT_EQ(0, firstCity.compare("York"));
	ASSERT_EQ(0, secondCity.compare("York"));
	ASSERT_EQ(0, thirdCity.compare("Unknown"));
}
#endif




