#ifdef __linux__

#include "stdafx.h"

#else

#include <string>
#include <exception>
#include <Isolator.h>
#include <gtest/gtest.h>
using namespace std;

#endif

#include "../ClassesUnderTest/Person.h"
#include "../ClassesUnderTest/GPSLocation.h"
#include "../ClassesUnderTest/ITransport.h"

// This sections contains examples about faking and asserting methods with out parameters
class FakingAnOutParams : public ::testing::Test
{
public:
	virtual void TearDown()
	{
		ISOLATOR_CLEANUP();
	}
};

#ifdef __linux__

class TransportImpl : public ITransport
{
public:
        virtual void SetTransport(char * name) {} 
        virtual string GetTransport() { return ""; } 
        virtual GPSLocation* GetLocation() { return NULL; } 
        // note: we are returing vector by value
        virtual vector<GPSLocation> GetAllStops() { return vector<GPSLocation>(); } 
        virtual void WriteAllStops(GPSLocation* locations) {} 
};

TEST_F(FakingAnOutParams, MakeITransportUsable)
{
        ITransport * it = new TransportImpl();
        delete it; 
}

#endif


/*	To set an out argument in a public method use RET with WHEN_CALLED.
    Note, we use BY_REF macro to assign a value to an out parameter.
*/
TEST_F(FakingAnOutParams, InPublicMethodCall)
{
	Person* person = new Person();
	WHEN_CALLED(person->GetOlder(RET(BY_REF(5))));

	int age = 0;
	person->GetOlder(&age);

	ASSERT_EQ(age, 5);
	delete person;
}

/*	This test shows the usage of the conditional WHEN_CALLED macro for a non public method called with an out arguments.
    This test also shows BY_REF usage - in condition and as return value for an out parameter.
*/
TEST_F(FakingAnOutParams, InPublicMethodCallWithCondition)
{	
	Person* person = new Person();
	WHEN_CALLED(person->GetOlder(RET_IF(EQ(BY_REF(5)), BY_REF(0))));

	int age = 5;
	person->GetOlder(&age);

	ASSERT_EQ(age, 0);
	delete person;
}

/*	To set an out argument in a non-public method use RET with PRIVATE_WHEN_CALLED.
*/
TEST_F(FakingAnOutParams, InPrivateMethodCall)
{
	Person* person = new Person();
	GPSLocation gpsLocationToReturn(9,9);
	PRIVATE_WHEN_CALLED(person, GetAddress, RET(&gpsLocationToReturn)).Return(true);

	GPSLocation gpsLocation(0,0);
	person->PublicGetAddress(gpsLocation);

	ASSERT_EQ(gpsLocation.Latitude(), 9);
	delete person;
}

/*
 * This test shows the usage of the conditional PRIVATE_WHEN_CALLED macro for a non public method called with an out arguments.
 * If person->ChangeLastName(char** otherLastName) is called with "Johns" it will return false, otherwise - true
*/
TEST_F(FakingAnOutParams, InPrivateMethodCallWithCondition)
{
	Person* person = FAKE<Person>(FakeOptions::CallOriginal); // Can also use FakeOptions::Lean, but methods will not be tracked and asserting them will require using WHEN_CALLED on the method

	char* out = "Smith";

	PRIVATE_WHEN_CALLED(person, ChangeLastName, RET_IF(EQ(BY_REF("Johns")), &out)).Return(false);
	
	Person* otherPerson = new Person();
	otherPerson->lastName = "Johns";

	ASSERT_FALSE(person->GetMarried(otherPerson));
	ASSERT_EQ(otherPerson->lastName, out);
	delete otherPerson;
}

/*
 * This test shows the usage of the conditional PRIVATE_WHEN_CALLED macro for a non public method called with an out arguments without an out argument assignment.
 * If person->ChangeLastName(char** otherLastName) is called with "Smith" it will return false, otherwise - true. But an out argument won't change.
*/
#if defined(__linux__) && !defined(__clang__)

struct MatchCharPtrPredicateSmith
{
    bool operator ()(char ** s) { return !strcmp(*s, "Smith"); }
};

#endif

TEST_F(FakingAnOutParams, InPrivateMethodCallWithConditionWithoutAssigment)
{
	Person* person = FAKE<Person>(FakeOptions::CallOriginal); // Can also use FakeOptions::Lean, but methods will not be tracked and asserting them will require using WHEN_CALLED on the method

#if defined(__linux__) && !defined(__clang__)
        PRIVATE_WHEN_CALLED(person, ChangeLastName, IS_REF<char**>(MatchCharPtrPredicateSmith())).Return(false);
#else
	PRIVATE_WHEN_CALLED(person, ChangeLastName, IS_REF<char**>([](char** s) {return !strcmp(*s, "Smith");})).Return(false);
#endif
	
	Person* otherPerson = new Person();
	otherPerson->lastName = "Smith";

	char* neededLastName = "Smith";
	ASSERT_FALSE(person->GetMarried(otherPerson));
	ASSERT_FALSE(strcmp(otherPerson->lastName, "Smith"));
	delete otherPerson;
}

/*
 * This test shows the usage of the RET macro for an out argument which needs additional size definition.
 * To assign an out GPSLocation[] array we need to pass a RET macro additional argument - allocation memory size of an array.
*/
TEST_F(FakingAnOutParams, AnArrayInPublicMethodCall)
{
	ITransport* transport = FAKE_ALL<ITransport>();
	GPSLocation locations[] = {GPSLocation(1, 1), GPSLocation(1, 0), GPSLocation(0, 1) };
	size_t sizeOfArray = ARRAYSIZE(locations);
	WHEN_CALLED(transport->WriteAllStops(RET(locations, sizeOfArray * sizeof(GPSLocation)))).Ignore();

	Person* person = new Person();
	ISOLATOR_SET_VARIABLE(person, m_transport, transport);
	GPSLocation personLocations[3];

	person->WriteAllTransportStops(personLocations);

	ASSERT_EQ(personLocations[0].Latitude(), 1);
	ASSERT_EQ(personLocations[0].Longitude(), 1);
}
