#ifdef __linux__

#include "stdafx.h"

#else

#include <string>
#include <exception>
#include <Isolator.h>
#include <gtest/gtest.h>
using namespace std;

#endif

#include "../ClassesUnderTest/Person.h"

// PRIVATE_WHEN_CALLED() is used to change behavior of non-public methods, usually on the tested object
class FakingNonPublicMethods : public ::testing::Test
{
public:
	virtual void TearDown()
	{
		ISOLATOR_CLEANUP();
	}
};


/**************************************************************************************
	Setting behavior on non-public methods with PRIVATE_WHEN_CALLED
***************************************************************************************/

/*	In this test, we fake the private IsValidName and check we get the 
	exception thrown from the Person class. 
*/
TEST_F(FakingNonPublicMethods, ChangingBehaviorOfPrivateMethod)
{
	Person* person = new Person();
	PRIVATE_WHEN_CALLED(person, IsValidName).Return(false);

	ASSERT_THROW(person->Save(""), exception);

	delete person;
}

/*	In this test, we fake the private overloaded method CanPing 
*/
TEST_F(FakingNonPublicMethods, ChangingBehaviorOfPrivateOverloadedMethod)
{
	Person* person = new Person();
	PRIVATE_WHEN_CALLED(person, CanPing, TYPEOF(bool)).Return(true);
	ASSERT_TRUE(person->Ping());
	delete person;
}


/*	To assert a call on a non-public use PRIVATE_ASSERT_WAS_CALLED.
*/
TEST_F(FakingNonPublicMethods, AssertingPrivateMethodCall)
{
	Person* person = new Person();

	PRIVATE_WHEN_CALLED(person, IsValidName).Return(true);
    PRIVATE_WHEN_CALLED(person, SaveAll).Ignore();

	person->Save("");
	PRIVATE_ASSERT_WAS_CALLED(person, SaveAll);
	delete person;
}

/*	To assert a call on a overloaded non-public use PRIVATE_ASSERT_WAS_CALLED.
*/
TEST_F(FakingNonPublicMethods, AssertingOverloadedPrivateMethodCall)
{
	Person* person = new Person();
	PRIVATE_WHEN_CALLED(person, CanPing, TYPEOF(bool)).Return(true);

	person->Ping();

	PRIVATE_ASSERT_WAS_CALLED(person, CanPing, TYPEOF(bool));
	delete person;
}

// If there is no lambda support you can use this predicate
/*
** gcc 4.4.7 with no c++11 support is used as basic version
*/

struct MatchBoolPredicate1
{
    bool operator ()(bool val) { return val; }
};

#ifdef NO_CPP_LAMBDA_SUPPORT
TEST_F(FakingNonPublicMethods, AssertingOverloadedPrivateMethodCallWithMatchingArgumetns)
{
	Person* person = new Person();
	PRIVATE_WHEN_CALLED(person, CanPing, TYPEOF(bool)).Ignore();

	person->Ping();

    PRIVATE_ASSERT_WAS_CALLED(person, CanPing, IS<bool>(MatchBoolPredicate1()));

	delete person;
}
#else
/*	To assert a call on a overloaded with matching arguments use IS<> macro
*/
TEST_F(FakingNonPublicMethods, AssertingOverloadedPrivateMethodCallWithMatchingArgumetns)
{
	Person* person = new Person();
	PRIVATE_WHEN_CALLED(person, CanPing, TYPEOF(bool)).Ignore();

	person->Ping();

	PRIVATE_ASSERT_WAS_CALLED(person, CanPing, IS<bool>([](bool x){return x;}));

	delete person;
}
#endif

/*	To set an out argument in a non-public method use RET with PRIVATE_WHEN_CALLED.
*/
TEST_F(FakingNonPublicMethods,OutParametersInPrivateMethodCall)
{
	Person* person = new Person();
	GPSLocation gpsLocationToReturn(9,9);
	PRIVATE_WHEN_CALLED(person, GetAddress, RET(&gpsLocationToReturn)).Return(true);

	GPSLocation gpsLocation(0,0);
	person->PublicGetAddress(gpsLocation);

	ASSERT_EQ(gpsLocation.Latitude(), 9);
	delete person;
}

/*
 * This test shows the usage of the conditional PRIVATE_WHEN_CALLED macro for a non public method called with an out arguments.
 * If person->ChangeLastName(char** otherLastName) is called with "Johns" it will return false, otherwise - true
*/
TEST_F(FakingNonPublicMethods, OutParametersInPrivateMethodCallWithPredicate)
{
	Person* person = FAKE<Person>(FakeOptions::CallOriginal); // Can also use FakeOptions::Lean, but methods will not be tracked and asserting them will require using WHEN_CALLED on the method

	char* out = "Smith";
	PRIVATE_WHEN_CALLED(person, ChangeLastName, RET_IF(EQ(BY_REF("Johns")), &out)).Return(false);
	
	Person* otherPerson = new Person();
	otherPerson->lastName = "Johns";
	
	ASSERT_FALSE(person->GetMarried(otherPerson));
	ASSERT_EQ(otherPerson->lastName, out);
	delete otherPerson;
}


